local ADDON_NAME="AVR"
local Core=AVR
local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME, true)

local function strsplit(str)
	local tbl = {}
	for v in string.gmatch(str, "[^ ]+") do
	  tinsert(tbl, v)
	end
	return tbl
end

local toolSettings={
	paint = {
		followPlayer = false,
		followRotation = false,
		detail = 0.5
	},
	newMesh = {
		circle = {
			radius = 10.0,
			segments = 30,
			lineWidth = 32,
			dashed = false,
			follow = true
		},
		filledCircle = {
			radius = 10.0,
			segments = 30,
			follow = false
		},
		cone = {
			angle = 30.0,
			radius = 10.0,
			segments = 6,
			lineWidth = 32,
			dashed = false,
			circle = true,
			follow = true
		},
		unitMarker = {
			unit = "target"
		},
		yardStick = {
			minHeight = 0,
			maxHeight = 10.0,
			vertical = false,
			div = 2
		},
		rangeWarning = {
			range = 10,
			radius = 1
		}
	}
}

local function getToolSettings(info)
	if type(info)=="string" then
		info={info}
	end
	for i=1,#info do
		if toolSettings[info[i]] then
			local t=toolSettings
			for j=i,#info do
				t=t[info[j]]
				if t==nil then return nil end
			end
			return t
		end
	end
	return nil
end

local function setToolSettings(info,val)
	if type(info)=="string" then
		info={info}
	end
	for i=1,#info do
		if toolSettings[info[i]] then
			local t=toolSettings
			for j=i,#info-1 do
				t=t[info[j]]
				if t==nil then return end
			end
			t[info[#info]]=val
		end
	end
	return nil
end

local function getMeshesArgs(scene)
	local ret={}
	local i,m
	for i,m in ipairs(scene.meshes) do
		ret[""..i]=m:GetOptions()
	end
	return ret
end

local function getSceneTable(scene)
	return {
		name = (scene.from and (scene.from..": "..scene.name) or scene.name),
		type = "group",
		get = function(info) 
			if type(info)~="string" then info=info[#info] end
			return scene[info]
		end,
		set = function(info,val)
			if type(info)~="string" then info=info[#info] end
			scene[info]=val
		end,
		args={
			name = {
				type = "input",
				name = L["Name"],
				order = 10,
				width = "full",
			},
			visible = {
				type = "toggle",
				name = L["Visible"],
				order = 15
			},
			select = {
				type = "toggle",
				name = L["Select"],
				order = 20,
				get =	function(_)
							if Core.sceneManager:GetSelectedScene()==scene then return true
							else return false end
						end,
				set =	function(_,val)
							if val then Core.sceneManager:SetSelectedScene(scene)
							else Core.sceneManager:SetSelectedScene(nil) end
						end
			},
			clear = {
				type = "execute",
				name = L["Clear scene"],
				order = 25,
				width = "full",
				confirm = true,
				func = function() scene:ClearScene() end
			},
			remove = {
				type = "execute",
				name = L["Remove scene"],
				order = 30,
				width = "full",
				confirm = true,
				func = function() scene:Remove() end
			},
			zone = {
				name = L["Zone"],
				type = "input",
				order = 40,
				set = 	function(_,val)
							val=string.gsub(string.gsub(val,"^%s*",""),"%s*$","")
							if string.len(val)==0 then val=nil end
							scene.zone=val
						end
				
			},
			currentZone = {
				name = L["Current zone"],
				type = "execute",
				order = 50,
				func = function() scene.zone=Core.zoneData:GetCurrentZone() end
			},
			sharing = {
				type = "group",
				name = L["Share"],
				order = 60,
				inline = true,
				args = {
					shareChannel = {
						name = L["Channel"],
						type = "select",
						order = 10,
						values = {RAID="Raid",PARTY="Party",GUILD="Guild",BATTLEGROUND="Battleground",WHISPER="Whisper"},
						set = 	function(_,val)
									if val~="WHISPER" then scene.shareTarget="" end
									scene.shareChannel=val
								end
					},
					shareTarget = {
						name = L["Whisper target"],
						type = "input",
						order = 20,
						set = 	function(_,val)
									if scene.shareChannel=="WHISPER" then scene.shareTarget=val end
								end
					},
					send = {
						name = L["Send"],
						type = "execute",
						order = 30,
						func =	function()
									Core:SendScene(scene,scene.shareChannel,scene.shareTarget)
								end
					}
				}
			},
			id = {
				name = L["Scene id"],
				type = "input",
				disabled = true,
				order = 70,
				set = function() end,
				get = function() return ''..scene.id end
			},
			from = {
				name = L["Owner"],
				type = "input",
				disabled = true,
				order = 80,
				set = function() end,
			},
			ownCopy = {
				name = L["Make own copy"],
				type = "execute",
				order = 90,
				disabled = (scene.from==nil),
				func = function() scene:MakeOwnCopy() end
			},
			paint = {
				type = "group",
				name = L["Paint"],
				order = 140,
				get = getToolSettings,
				set = setToolSettings,
				args = {
					followPlayer = { name=L["Follow player"], type = "toggle" },
					followRotation = { name=L["Follow rotation"], type = "toggle" },
					detail = { 
						name = L["Detail level"],
						type = "range",
						min = 0.1,
						max = 5.0,
						bigStep = 0.1,
					},
					start = { 
						type = "execute",
						name = L["Start"],
						func = function()
							Core.mousePaint:EnableDraw(true,toolSettings.paint.followPlayer,toolSettings.paint.followRotate,scene)
							Core.mousePaint:SetDetailLevel(toolSettings.paint.detail)
						end
					}
				}
			},
			meshes = {
				type = "group",
				name = L["Meshes"],
				order = 150,
				args = getMeshesArgs(scene)
			},
			newMesh = {
				type = "group",
				name = L["Add new mesh"],
				order = 160,
				get = getToolSettings,
				set = setToolSettings,
				args = {
					compass = {
						name = L["Compass"],
						type = "group",
						args = {
							add = {
								name = L["Add"],
								order = 10,
								type = "execute",
								func = 	function()
											scene:AddMesh(AVRCompass())
										end
							}
						}
					},
					circle = {
						name = L["Circle"],
						type = "group",
						args = {
							add = {
								name = L["Add"],
								order = 10,
								type = "execute",
								func = 	function()
											scene:AddMesh(AVRCircle(
												toolSettings.newMesh.circle.radius,
												toolSettings.newMesh.circle.segments,
												toolSettings.newMesh.circle.dashed,
												toolSettings.newMesh.circle.lineWidth
											)):SetFollowPlayer(true)
											if not toolSettings.newMesh.circle.follow then
												scene:GetLastMesh():Detach()									
											end
										end
							},
							radius = {
								name = L["Radius"],
								order = 20,
								type = "range",
								min = 0.0,
								max = 500.0,
								bigStep = 1.0
							},
							follow = {
								name = L["Follow player"],
								order = 25,
								type = "toggle"
							},
							lineWidth = {
								name = L["Line width"],
								order = 30,
								type = "range",
								min = 0,
								max = 255,
								bigStep = 1,
								step = 1
							},
							segments = {
								name = L["Segments"],
								order = 40,
								type = "range",
								min = 3,
								max = 200,
								bigStep = 1,
								step = 1,
							},
							dashed = {
								name = L["Dashed"],
								type = "toggle",
								order = 50
							},
						}
					},
					filledCircle = {
						name = L["Filled circle"],
						type = "group",
						args = {
							add = {
								name = L["Add"],
								order = 10,
								type = "execute",
								func = 	function()
											scene:AddMesh(AVRFilledCircle(
												toolSettings.newMesh.filledCircle.radius,
												toolSettings.newMesh.filledCircle.segments
											):SetFollowPlayer(true))
											if not toolSettings.newMesh.filledCircle.follow then
												scene:GetLastMesh():Detach()									
											end
										end
							},
							radius = {
								name = L["Radius"],
								order = 20,
								type = "range",
								min = 0.0,
								max = 500.0,
								bigStep = 1.0
							},
							follow = {
								name = L["Follow player"],
								order = 25,
								type = "toggle"
							},
							segments = {
								name = L["Segments"],
								order = 40,
								type = "range",
								min = 3,
								max = 200,
								bigStep = 1,
								step = 1,
							},
						}
					},
					cone = {
						name = L["Cone"],
						type = "group",
						args = {
							add = {
								name = L["Add"],
								order = 10,
								type = "execute",
								func = 	function()
											scene:AddMesh(AVRCone(
												toolSettings.newMesh.cone.radius,
												toolSettings.newMesh.cone.angle,
												toolSettings.newMesh.cone.segments,
												toolSettings.newMesh.cone.dashed,
												toolSettings.newMesh.cone.circle,
												toolSettings.newMesh.cone.lineWidth
											)):SetFollowPlayer(true)
											if not toolSettings.newMesh.cone.follow then
												scene:GetLastMesh():Detach()									
											end
										end
							},
							radius = {
								name = L["Radius"],
								order = 20,
								type = "range",
								min = 0.0,
								max = 500.0,
								bigStep = 1.0
							},
							angle = {
								name = L["Angle"],
								order = 23,
								type = "range",
								min = 1.0,
								max = 360.0,
								bigStep = 1.0
							},
							follow = {
								name = L["Follow player"],
								order = 25,
								type = "toggle"
							},
							lineWidth = {
								name = L["Line width"],
								order = 30,
								type = "range",
								min = 0,
								max = 255,
								bigStep = 1,
								step = 1
							},
							segments = {
								name = L["Segments"],
								order = 40,
								type = "range",
								min = 3,
								max = 200,
								bigStep = 1,
								step = 1,
							},
							dashed = {
								name = L["Dashed"],
								type = "toggle",
								order = 50
							},
							circle = {
								name = L["Circle"],
								type = "toggle",
								order = 60
							}
						}
					},					
					player = {
						name = L["Player marker"],
						type = "group",
						args = {
							add = {
								type = "execute",
								name = "Add",
								func =	function()
											scene:AddMesh(AVRPlayerMarker())
										end
							}
						}
					},
					vargoth = {
						name = L["Archmage Vargoth spawn point"],
						type = "group",
						args = {
							add = {
								type = "execute",
								name = L["Add"],
								func =	function()
											scene:AddMesh(AVRVargoth())
										end
							}
						}
					},
					blink = {
						name = L["Blink location"],
						type = "group",
						args = {
							add = {
								type = "execute",
								name = L["Add"],
								order = 10,
								func =	function()
											scene:AddMesh(AVRBlink())
										end
							}
						}
					},
					crosshair = {
						name = L["Crosshair"],
						type = "group",
						args = {
							add = {
								type = "execute",
								name = L["Add"],
								func =	function()
											scene:AddMesh(AVRPositionCrosshair())
										end
							}
						}
					},
					unitMarker = {
						name = L["Unit marker"],
						type = "group",
						args = {
							add = {
								type = "execute",
								name = L["Add"],
								func = 	function()
											scene:AddMesh(AVRUnitMarker(toolSettings.newMesh.unitMarker.unit))
										end
							},
							unit = {
								name = L["Unit"],
								type = "input",
								order = 20
							},
						}
					},
					yardStick = {
						name = L["Yard stick"],
						type = "group",
						args = {
							add = {
								type = "execute",
								name = L["Add"],
								order = 10,
								func = 	function()
											scene:AddMesh(AVRYardStick(toolSettings.newMesh.yardStick.maxHeight,
																		 toolSettings.newMesh.yardStick.minHeight,
																		 toolSettings.newMesh.yardStick.vertical,
																		 toolSettings.newMesh.yardStick.div))
										end
							},
							minHeight = {
								name = L["Min"],
								type = "range",
								order = 20,
								min = -100.0,
								max = 100.0,
								bigStep = 0.1
							},
							maxHeight = {
								name = L["Max"],
								type = "range",
								order = 30,
								min = -100.0,
								max = 100.0,
								bigStep = 0.1
							},
							div = {
								name = L["Divisions"],
								type = "range",
								order = 40,
								min = 1,
								max = 100,
								bigStep = 1
							},
							vertical = {
								name = L["Vertical"],
								type = "toggle",
								order = 50,
							},
						}
					},
					rangeWarning = {
						name = L["Range warning"],
						type = "group",
						args = {
							add = {
								name = L["Add"],
								order = 10,
								type = "execute",
								func = 	function()
											scene:AddMesh(AVRRangeWarning(
												toolSettings.newMesh.rangeWarning.range,
												toolSettings.newMesh.rangeWarning.radius
											))
										end
							},
							range = {
								name = L["Range"],
								order = 20,
								type = "range",
								min = 0.0,
								max = 500.0,
								bigStep = 1.0
							},
							radius = {
								name = L["Radius"],
								order = 30,
								type = "range",
								min = 0.0,
								max = 10.0,
								bigStep = 0.5
							},
						}
					}
				}
			}, -- end newMesh
		}
	}
end

local function getScenesArgs() 
	if not Core.sceneManager then return {} end
	
	local ss=Core.sceneManager.scenes
	local ret={
		newScene = {
			type = "execute",
			name = L["New scene"],
			func = 	function()
						if Core.sceneManager then Core.sceneManager:NewScene() end
					end
		}
	}
	for i,scene in ipairs(ss) do
		ret[""..i]=getSceneTable(scene)
	end
	return ret
end

local function setOptionParam(info,value,folder)
	local varName = nil
	if type(info) == "string" then
		varName = info
	else
		varName = info[#info]
	end
	if not folder then folder=Core.db.profile end
	folder[varName] = value
end
local function getOptionParam(info,folder)
	local varName = nil
	if type(info) == "string" then
		varName = info
	else
		varName = info[#info]
	end
	if not folder then folder=Core.db.profile end
	return folder[varName]
end

Core.options = {
    name = ADDON_NAME,
    handler = Core,
    type = 'group',
	get = getOptionParam,
	set = setOptionParam,
}
local rootArgs=nil
setmetatable(Core.options,{
	__index=function(table,key)
		if key~="args" then return nil end
		if rootArgs~=nil then return rootArgs end
		rootArgs={
			menu = {
				type="execute",
				name=L["Menu"],
				guiHidden=true,
				func = function() Core.AceConfigDialog:Open(ADDON_NAME) end
			},
			general = {
				type="group",
				name= L["General"],
				order = 10,
				args={
					enable = {
						type = "toggle",
						name = L["Enable"],
						desc = L["Enable or disable addon"],
						width = "full",
						order = 0,
						set = function(info, v)
							setOptionParam(info,v)
							if v then
								Core:DoEnable()
							else
								Core:DoDisable()
							end
						end,
					},
					hideAll = {
						type = "toggle",
						name = L["Hide all"],
						desc = L["Hide all 3d objects"],
						width = "full",
						order = 1
					}
				},
			},
			sharing = {
				type = "group",
				name = L["Sharing"],
				order = 20,
				args = {
					acceptFrom = {
						type = "group",
						inline = true,
						name = L["Accept incoming scenes from"],
						args = {
							acceptScenesFromRaid = {
								type="toggle",
								name=L["Raid"],
							},
							acceptScenesFromParty = {
								type="toggle",
								name=L["Party"],
							},
							acceptScenesFromBG = {
								type="toggle",
								name=L["Battleground"],
							},
							acceptScenesFromGuild = {
								type="toggle",
								name=L["Guild"],
							},
							acceptScenesFromWhisper = {
								type="toggle",
								name=L["Whisper"],
							}
						}
					},
					receiveOwnScenes = {
						type="select",
						name=L["Receive own scenes"],
						values = {yes=L["Yes"],no=L["No"],hidden=L["As hidden"]}
					},
				}
			}
			-- scenes
			-- scene
			-- mesh
		}
		if Core.sceneManager then
			rootArgs.scenes={
				type = "group",
				name = L["Scenes"],
				order = 20,
				args = getScenesArgs()
			}
			
			local scene=Core.sceneManager:GetSelectedScene()
			if scene then
				rootArgs.scene=getSceneTable(scene)
				rootArgs.scene.guiHidden=true
				rootArgs.scene.order=30
				local mesh=scene:GetSelectedMesh()
				if mesh then
					rootArgs.mesh=mesh:GetOptions()
					rootArgs.mesh.guiHidden=true
					rootArgs.mesh.order=40
				end
			end
		end
		return rootArgs
	end
})

function Core:OptionsChanged()
	rootArgs=nil
	Core.AceConfigRegistry:NotifyChange(ADDON_NAME)
end



Core.defaultOptions = {
	profile = {
		enable = true,
		hideAll = false,
		scenes = {},
		sceneid = 1,
		selectedScene = 0,
		noPackDoubles = false,
		receiveOwnScenes = "hidden",
		acceptScenesFromRaid = true,
		acceptScenesFromParty = true,
		acceptScenesFromBG = true,
		acceptScenesFromGuild = true,
		acceptScenesFromWhisper = true,
	}
}

function Core:OpenLDBMenu()
end
